-module(gLuaGen).

-export([
   genLua/4
]).

spellClassHead(MsgName, MsgId) ->
   <<"\tpublic class ", MsgName/binary, " : ProtocolCore.ISerialize, ProtocolCore.IDeserialize<", MsgName/binary, ">\n\t{\n\t\tpublic const int ProtoId = ", (integer_to_binary(MsgId))/binary, ";\n">>.

spellClassMember(FieldList) ->
   <<<<(gCsField:builtRecStr(OneTypeName))/binary>> || OneTypeName <- FieldList>>.

spellCalssDSTem() ->
   <<"\n\t\tpublic byte[] Serialize()\n\t\t{\n\t\t\tusing var memoryStream = new MemoryStream();
\t\t\tusing var binaryWriter = new BinaryWriter(memoryStream);
\t\t\tSerialize(binaryWriter);
\t\t\treturn memoryStream.ToArray();
\t\t}

\t\tpublic void Deserialize(byte[] data)
\t\t{
\t\t\tusing var memoryStream = new MemoryStream(data);
\t\t\tusing var binaryReader = new BinaryReader(memoryStream);
\t\t\tDeserialize(binaryReader);
\t\t}\n">>.

spellCalssSerialize(FieldList) ->
   FunHead = <<"\t\tpublic void Serialize(BinaryWriter binaryWriter)\n\t\t{\n">>,
   FunBody = <<<<"\t\t\tbinaryWriter.WriteValue(", NameStr/binary, ");\n">> || {_TypeStr, NameStr} <- FieldList>>,
   <<FunHead/binary, FunBody/binary, "\t\t}\n">>.

spellCalssDeserialize(FieldList) ->
   FunHead = <<"\t\tpublic void Deserialize(BinaryReader binaryReader)\n\t\t{\n">>,
   FunBody = <<<<"\t\t\tbinaryReader.ReadValue(out ", NameStr/binary, ");\n">> || {_TypeStr, NameStr} <- FieldList>>,
   <<FunHead/binary, FunBody/binary, "\t\t}\n">>.

spellClassEnd() ->
   <<"\t}\n">>.

genLua(SortedSProtoList, _SortedErrList, _HrlDir, CSDir) ->
   FunSpell =
      fun({MsgName, MsgId, FieldList}, ClassBinAcc) ->
         H = spellClassHead(MsgName, MsgId),
         M = spellClassMember(FieldList),
         DS = spellCalssDSTem(),
         S = spellCalssSerialize(FieldList),
         D = spellCalssDeserialize(FieldList),
         E = spellClassEnd(),
         <<ClassBinAcc/binary, H/binary, M/binary, DS/binary, S/binary, D/binary, E/binary>>
      end,
   LastClassBinAcc = lists:foldl(FunSpell, <<>>, SortedSProtoList),

   %% todo error code
   %% ErrCodeStr = spellErrCodeHrl(SortedErrList, <<>>),

   CSEndStr = protoEnd(),
   OutputHrlStr = <<LastClassBinAcc/binary, CSEndStr/binary>>,
   CSFilename = do_write_cs(CSDir, protoMsg, OutputHrlStr),

   io:format("protoConvert erl dir : ~s ~n", [CSDir]),
   io:format("protoConvert to erl file ~s succ.~n", [CSFilename]),
   ok.

do_write_cs(OutDir, Mod, BinStr) ->
   Filename = filename:join([OutDir, atom_to_list(Mod) ++ ".cs"]),
   case file:write_file(Filename, BinStr) of
      ok ->
         ok;
      _Ret ->
         io:format("write to cs file error:~p ~n", [_Ret])
   end,
   Filename.